﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using MindFusion.Charting;
using MindFusion.Drawing;

using Color = System.Drawing.Color;
using MindFusion.Charting.Components;


namespace LineChart
{
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();

			// create line brushes
			firstBrush = new SolidBrush(Color.SkyBlue);
			secondBrush = new SolidBrush(Color.DeepPink);
			thirdBrush = new SolidBrush(Color.Green);


			lineChart.LegendRenderer.Background = new SolidBrush(Color.Khaki);
			lineChart.XAxisLabelRotationAngle = 45;

			// create sample data series
			lineChart.Series.Add(
				new Series2D(
					new List<double> { 0,1,2,3,4,5,6,7,8,9,10,11 },
					new List<double> { 1,2,3,4,5,6,7,8,9,10,11,12 },
					labels
				)
				{ Title = "Series 1", SupportedLabels = LabelKinds.XAxisLabel });

			/*lineChart.Series.Add(
				new Series2D(
					new List<double> { 0,1,2,3,4,5,6,7,8,9,10,11 },
					new List<double> { 2,4,6,8,10,12,14,16,18,20,22,24 },
					labels
				)
				{ Title = "Series 2" });*/

			lineChart.Series.Add(
				new FunctionSeries("x*Sin(x)+x+5",12, 12)
				{ Title = "Series 3" });

			lineChart.XAxis.Interval = 1;
			lineChart.ShowScatter = true;
			// assign one brush per series
			lineChart.Plot.SeriesStyle = new MixedSeriesStyle()
			{
				CommonFills = new List<Brush>()
				{
					firstBrush, secondBrush, thirdBrush
				},
				CommonStrokes = new List<Brush>()
				{
					firstBrush, secondBrush, thirdBrush
				},
				UniformStrokeThickness = 15
			};

            lineChart.KeyDown += new KeyEventHandler(lineChart_KeyDown);
            lineChart.KeyUp += new KeyEventHandler(lineChart_KeyUp);

			// set up grid type combo box
			cbxGridType.Items.Add(GridType.Crossed);
			cbxGridType.Items.Add(GridType.Horizontal);
			cbxGridType.Items.Add(GridType.None);
			cbxGridType.Items.Add(GridType.Vertical);
			cbxGridType.SelectedIndex = 0;
			lineChart.GridType = GridType.Crossed;

			// set up line type combo box
			cbxLineType.Items.Add(LineType.Polyline);
			cbxLineType.Items.Add(LineType.Step);
			cbxLineType.Items.Add(LineType.Curve);
			cbxLineType.SelectedIndex = 0;
			lineChart.LineType = LineType.Polyline;

			/*StackPanel yStack = null;
			foreach (var component in lineChart.ChartPanel.Children)
			{
				if (component.GridRow == 0 &&
					component.GridColumn == 0 &&
					component is StackPanel)
				{
					yStack = (StackPanel)component;
					break;
				}
			}
			yStack.Children.Add(
				new RangeSelector(
					lineChart.YAxis, -100, 100) { Orientation = MindFusion.Charting.Components.Orientation.Vertical });*/

			lineChart.ShowYRangeSelector = true;
			lineChart.YScrollRangeMin = -100;
			lineChart.YScrollRangeMax = 100;

			lineChart.ShowXRangeSelector = true;
			lineChart.XScrollRangeMin = -100;
			lineChart.XScrollRangeMax = 100;

			lineChart.ShowScatter = true;
			var scatterRenderer = lineChart.Plot.
				SeriesRenderers.OfType<ScatterRenderer>().FirstOrDefault();
			if (scatterRenderer != null)
			{
				scatterRenderer.Shape = ScatterType.Diamond;
				scatterRenderer.ShapeSize = 25;
			}

            var axis = new Axis { MinValue = 0, MaxValue = 200 };

            var y2Axis = new YAxisRenderer(axis, lineChart.XAxis);
            y2Axis.GridColumn = 2;
            y2Axis.PlotLeftSide = false;
            y2Axis.GridRow = 0;
            y2Axis.LabelsSource = lineChart.Plot;

            lineChart.ChartPanel.Columns.Add(new GridColumn());
            lineChart.ChartPanel.Children.Add(y2Axis);

            var y2Series = new ObservableCollection<Series>();
            y2Series.Add(
                new Series2D(
					new List<double> { 0,1,2,3,4,5,6,7,8,9,10,11 },
					new List<double> { 2,4,6,8,10,12,14,16,18,20,22,24 },
					labels
				)
				{ Title = "Series 2" });
            var y2Renderer = new LineRenderer(y2Series);
            y2Renderer.YAxis = axis;
            lineChart.Plot.SeriesRenderers.Add(y2Renderer);

            var renderer = new MyRenderer();
            renderer.Series.Add(new Series3D(
                    new List<double> { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11 },
                    new List<double> { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12 },
                    new List<double> { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12 },
                    null
                ));
            lineChart.Plot.SeriesRenderers.Add(renderer);
        }

        void lineChart_KeyUp(object sender, KeyEventArgs e)
        {
            var plot = (Plot2D)lineChart.Plot;
            plot.VerticalScroll = false;
        }

        void lineChart_KeyDown(object sender, KeyEventArgs e)
        {
            var plot = (Plot2D)lineChart.Plot;
            plot.VerticalScroll = e.Shift;
        }

		void chbShowXTicks_CheckedChanged(object sender, EventArgs e)
		{
			lineChart.ShowXTicks = !lineChart.ShowXTicks;
			lineChart.Invalidate();
		}

		void chbShowYTicks_CheckedChanged(object sender, EventArgs e)
		{
			lineChart.ShowYTicks = !lineChart.ShowYTicks;
			lineChart.Invalidate();
		}

		void chbShowXCoords_CheckedChanged(object sender, EventArgs e)
		{
			lineChart.ShowXCoordinates = !lineChart.ShowXCoordinates;
			lineChart.Invalidate();
		}

		void chbShowYCoords_CheckedChanged(object sender, EventArgs e)
		{
			lineChart.ShowYCoordinates = !lineChart.ShowYCoordinates;
			lineChart.Invalidate();
		}

		void chbShowLegend_CheckedChanged(object sender, EventArgs e)
		{
			lineChart.ShowLegend = !lineChart.ShowLegend;
			lineChart.Invalidate();
		}

		void chbShowDataLabels_CheckedChanged(object sender, EventArgs e)
		{
			if (chbShowDataLabels.Checked)
				lineChart.ShowDataLabels = LabelKinds.All;
			else
				lineChart.ShowDataLabels = LabelKinds.None;
			lineChart.Invalidate();
		}

		void tbXAxisMax_Scroll(object sender, EventArgs e)
		{
			lineChart.XAxis.MaxValue = tbXAxisMax.Value;
			lineChart.Invalidate();
		}

		void tbYAxisMax_Scroll(object sender, EventArgs e)
		{
			lineChart.YAxis.MaxValue = tbYAxisMax.Value;
			lineChart.Invalidate();
		}

		void cbxGridType_SelectedValueChanged(object sender, EventArgs e)
		{
			lineChart.GridType = (GridType)cbxGridType.SelectedItem;
			lineChart.Invalidate();
		}

		void tbXAxisMin_Scroll(object sender, EventArgs e)
		{
			lineChart.XAxis.MinValue = tbXAxisMin.Value;
			lineChart.Invalidate();
		}

		void tbYAxisMin_Scroll(object sender, EventArgs e)
		{
			lineChart.YAxis.MinValue = tbYAxisMin.Value;
			lineChart.Invalidate();
		}

		void chbScrollGrid_CheckedChanged(object sender, EventArgs e)
		{
			lineChart.PinGrid = !lineChart.PinGrid;
		}

		void cbxLineType_SelectedValueChanged(object sender, EventArgs e)
		{
			lineChart.LineType = (LineType)cbxLineType.SelectedItem;
			lineChart.Invalidate();
		}

		List<string> labels = new List<string>()
		{
			"one", "two", "three", "four", "five", "six",
			"seven", "eight", "nine", "ten", "eleven", "twelve"
		};

		SolidBrush firstBrush;
		SolidBrush secondBrush;
		SolidBrush thirdBrush;

        void button1_Click(object sender, EventArgs e)
        {
            lineChart.Series.Clear();

            lineChart.Series.Add(
                new Series2D(
                    new List<double> { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11 },
                    new List<double> { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12 },
                    labels
                ) { Title = "new series", SupportedLabels = LabelKinds.XAxisLabel });
        }

        private void lineChart_DataItemClicked(object sender, HitResult e)
        {

        }

        private void lineChart_MouseClick(object sender, MouseEventArgs e)
        {
            var result = lineChart.NearestElement(e.Location);
            if (result != null)
                MessageBox.Show(result.Series + " " + result.Index + " " + result.Distance);
        }
	}

    class MyRenderer : Renderer2D
    {
        public MyRenderer() :
            base(new ObservableCollection<Series>())
        {
        }

        /// <summary>
        /// SeriesRenderer.Draw override. Draws bubbles representing data items in specified RenderContext.
        /// </summary>
        /// <param name="context">A RenderContext instance.</param>
        protected override void Draw(RenderContext context)
        {
            EnumVisiblePoints(context, (s, i, point) =>
            {
                var d = (float)Series[s].GetValue(i, 2) * 2;
                var color = d < 7 ? Color.Red : Color.Green;
                var rect = new System.Drawing.RectangleF(
                    point.X - d / 2, point.Y - d / 2, d, d);

                using (var brush = new System.Drawing.SolidBrush(color))
                    context.Graphics.FillRectangle(brush, rect);
            });
        }
    }
}